
# q-expansion of newform 836.2.a.d, downloaded from the LMFDB on 23 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 836
weight = 2
poly_data = [27, -60, 16, 28, -12, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0], 1], [[-5, 0, 1, 0, 0, 0], 1], [[-3, 28, -5, -12, 1, 1], 3], [[48, -43, -25, 15, 2, -1], 3], [[42, -50, -20, 24, 1, -2], 3]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0, 0, 0, 0], [0, -1, 0, 0, 0, 0], [0, 0, 0, 0, 0, -1], [0, 0, 0, 0, 1, 0], [1, 0, 0, 0, 0, 0], [0, 0, 0, 1, 0, 0], [3, -2, 0, 0, -1, -1], [1, 0, 0, 0, 0, 0], [0, -2, -1, 0, 0, -1], [0, 1, -1, 0, 0, 1], [-4, 0, -1, 1, 0, 1], [3, 2, 1, 0, 1, 2], [-1, 2, 0, 1, -1, -1], [2, -2, -1, 0, 0, 0], [2, 0, 0, 2, 0, 0], [2, -2, -2, 0, 0, 0], [-3, 2, -1, 0, -1, 0], [3, 0, 2, -2, 1, -1], [-2, 0, 1, -1, 2, -1], [-6, 3, 0, 0, 0, 0], [4, -2, -2, 0, 0, 2], [1, -4, -2, 0, -1, -3], [-3, 0, -1, 2, -1, 1], [-3, 0, -1, 0, 1, -2], [-7, 0, 1, 0, -1, 0], [-1, 0, 0, 0, -1, -3], [-3, 1, 1, 0, -1, 0], [2, 2, 2, -2, 0, -2], [6, 2, 0, 2, 0, 2], [-3, 0, 1, -2, 1, 2], [1, -2, 2, -2, 1, 1], [1, -2, 0, 2, -2, -1], [-5, -2, -3, 0, 0, 0], [5, 2, -1, 0, -1, -1], [-8, 0, 0, 2, 2, 2], [-4, 2, 4, 0, 0, 0], [-8, 2, -2, 2, 0, 3], [4, 2, 0, -2, 2, 0], [-9, 2, -2, 2, 1, 3], [2, -5, 1, 0, 0, -1], [-6, 4, -1, 1, 2, 1], [11, -4, 1, -2, -1, 0], [-8, 2, -1, 2, -2, -1], [1, -1, 1, -2, 1, -2], [-3, 0, -2, -2, 1, -1], [0, 4, 2, 0, -2, 2], [-1, 0, 0, 2, 1, -3], [-1, 2, 3, 0, 1, -2], [3, -4, 0, 2, -1, -1], [1, 0, 1, 0, -2, -2], [1, -4, 2, -2, -1, 1], [6, 8, -1, 2, 0, 2], [3, -3, -3, -2, -3, -2], [-6, 2, 3, 2, 2, 3], [-10, -4, 2, -2, 0, 0], [4, 0, -1, -2, -2, 0], [8, 0, -4, 0, 0, -2], [-3, 0, 0, -2, 0, -1], [14, 4, 4, 0, -2, 0], [1, -3, 1, 2, 1, 2], [6, 2, 0, -4, 3, 2], [-2, 0, -2, -1, -2, -2], [-3, 2, -2, -2, 3, 5], [-2, 4, 4, -2, -2, 0], [12, 0, 0, 4, 0, -1], [-5, 0, 1, -2, -1, 2], [-10, -5, -4, 0, 0, -4], [4, 0, 0, -1, 0, -2], [3, -6, 0, -2, -1, 1], [-7, 4, 2, 0, 3, 3], [6, 0, -3, -2, -4, -1], [3, 0, -4, -2, 0, 1], [-8, -2, -1, 4, -2, -1], [11, 1, 3, -2, 1, 0], [0, -4, -1, 1, -4, 1], [-4, -4, -1, -3, 4, 3], [4, 10, -2, 2, 0, 5], [-11, 2, 0, 2, 4, 3], [0, -4, 0, 2, 2, 2], [6, 3, 3, -4, 2, -1], [4, -10, -4, 0, 0, 0], [6, -6, -2, -2, 2, 2], [-6, 4, 0, 0, 2, 0], [15, -8, -1, -2, -3, -2], [-7, -2, -4, -4, 1, 1], [2, -6, -3, 0, -4, -3], [-13, -2, -3, 2, 5, 4], [7, 2, -4, 2, -1, 1], [-10, -2, 0, 2, -4, 0], [-2, 8, -1, 2, 2, 5], [6, -6, 1, -2, -2, 1], [10, 4, 2, 0, -5, 0], [0, -5, 2, 0, 4, 2], [10, 0, -2, 0, 3, -4], [0, 0, 2, 0, -4, -6], [0, -2, 1, 0, -2, -2], [7, 0, 5, -4, 3, -2], [-7, -6, -3, -2, 1, -6], [0, 4, 0, -2, 2, -4], [13, -8, 2, -4, 3, 5], [5, -4, 4, 0, -3, -5], [-6, 4, 2, 0, 0, -4], [13, -2, 0, 0, -3, -3], [12, 0, 0, 3, 0, -4], [4, 10, 0, 0, 1, 0], [-17, 4, -3, 0, -2, 4], [-14, 0, -4, 0, -4, 0], [-10, -2, 2, 4, -2, -4], [5, -1, -1, 0, 1, 0], [-4, 4, 2, 1, 8, 2], [-27, 8, 4, 0, 1, 1], [7, -2, 0, 0, 1, 1], [15, 4, 0, 2, 0, 1], [12, 4, -3, -2, 2, 3], [8, -4, 1, 0, 6, 7], [1, -8, -1, 0, 1, -2], [6, -12, 1, -2, -4, -3], [-4, -8, -3, 0, -2, -9], [-9, 4, 3, 0, -4, 2], [8, -4, 2, -2, 0, -6], [9, -8, -3, 6, -3, -4], [-1, 6, 0, 7, -1, 1], [-7, -1, 1, -2, -5, 0], [-12, -4, 0, 0, -2, -4], [4, 0, 5, -4, -2, 1], [8, 4, 6, 2, 2, 2], [7, 2, 5, 0, 4, 0], [-4, -2, 1, -4, 4, -3], [-10, -10, 3, -4, 2, -3], [-1, -10, -8, 0, 3, -3], [7, 4, -3, -2, -3, -1], [-5, 6, -6, 4, -3, 7], [-11, -2, -3, 0, -5, -6], [0, 0, 5, -2, 4, -2], [-6, -4, -4, -2, 4, 2], [18, 6, 4, 2, -2, 4], [0, -2, 0, 0, 2, 0], [6, 0, 2, -2, -2, -6], [11, -8, 3, -2, -3, -8], [-6, -4, -4, 2, 0, 2], [3, -6, 2, 0, 5, 5], [-7, -4, 4, 2, 3, 3], [-8, -2, -1, -2, -2, -5], [11, 4, -6, 2, -1, 1], [11, -2, 1, -2, 9, 6], [-18, 3, -2, 0, 2, 4], [0, -12, -2, 2, -2, -6], [-8, -4, -2, 1, 0, 4], [0, -8, -5, 6, -4, -1], [-9, 2, 2, 5, 3, -1], [9, -3, 5, -2, -5, -4], [17, -2, 3, -4, 2, 4], [6, -10, -2, 4, 0, -2], [-14, 2, 4, 0, -2, 2], [8, 0, 8, -4, 2, 0], [8, 4, 0, -4, 2, -4], [-17, 6, 3, 0, 5, -3], [5, 2, 2, -2, 0, -3], [-15, 2, 2, 0, 3, 7], [1, 0, 6, -4, -1, 1], [6, 6, -1, 8, -4, 1], [-9, 8, 6, -4, 5, -1], [17, 6, 4, -6, 5, 3], [-12, 1, 2, 0, -6, 4], [-9, 6, 1, -4, 3, 2], [2, 0, 1, -5, 6, 3], [-3, 2, -2, -1, -7, -3], [-17, 8, -8, 2, -3, 5]]
